@@ -329,8 +329,8 @@ class NcmClient:
         call_type = 'Accounts'
         get_url = '{0}/accounts/'.format(self.base_url)
 
-        allowed_params = ['account', 'account__in', 'id', 'id__in', 'name',
-                          'name__in', 'expand', 'limit', 'offset']
+        allowed_params = ['account', 'account__in', 'fields', 'id', 'id__in',
+                          'name', 'name__in', 'expand', 'limit', 'offset']
         params = self.__parse_kwargs(kwargs, allowed_params)
 
         return self.__get_json(get_url, call_type, params=params)
@@ -618,10 +618,7 @@ class NcmClient:
                                                      fields='configuration')[0]
 
         """Strip passwords which aren't stored in plain text"""
-        src_config = json.dumps(src_config).replace(', "wpapsk": "*"',
-                                                    '').replace(
-            '"wpapsk": "*"', '') \
-            .replace(', "password": "*"', '').replace('"password": "*"', '')
+        src_config = json.dumps(src_config).replace(', "wpapsk": "*"','').replace('"wpapsk": "*"', '').replace(', "password": "*"', '').replace('"password": "*"', '')
 
         """Get destination router Configuration Manager ID"""
         dst_config_man_id = \
@@ -1489,9 +1486,9 @@ class NcmClient:
         get_url = '{0}/routers/'.format(self.base_url)
 
         allowed_params = ['account', 'account__in', 'device_type',
-                          'device_type__in', 'group', 'group__in', 'id',
-                          'id__in', 'ipv4_address', 'ipv4_address__in', 'mac',
-                          'mac__in', 'name', 'name__in',
+                          'device_type__in', 'fields', 'group', 'group__in',
+                          'id', 'id__in', 'ipv4_address', 'ipv4_address__in',
+                          'mac', 'mac__in', 'name', 'name__in',
                           'reboot_required', 'reboot_required__in', 'state',
                           'state__in', 'state_updated_at__lt',
                           'state_updated_at__gt', 'updated_at__lt',
@@ -1641,7 +1638,7 @@ class NcmClient:
         :param lan_ip: LAN IP Address. (e.g. 192.168.1.1)
         :param netmask: Subnet mask. (e.g. 255.255.255.0)
         :param network_id: The ID of the network to update.
-          Numbering starts from 0.
+          Numbering starts from 0. Defaults to Primary LAN.
         :return:
         """
         call_type = 'LAN IP Address'
@@ -1728,3 +1725,146 @@ class NcmClient:
         ncm = self.session.put(put_url, data=json.dumps(put_data))
         result = self.__return_handler(ncm.status_code, ncm.json(), call_type)
         return result
+
+    def set_admin_password(self, router_id: int, new_password: str):
+        """
+        This method sets the local admin password for a router.
+        :param router_id: ID of router to update
+        :param new_password: Cleartext password to assign
+        :return:
+        """
+        call_type = 'Admin Password'
+
+        response = self.session.get(
+            '{0}/configuration_managers/?router.id={1}&fields=id'.format(
+                self.base_url,
+                str(router_id)))  # Get Configuration Managers ID
+        response = json.loads(response.content.decode(
+            "utf-8"))  # Decode the response and make it a dictionary
+        config_man_id = response['data'][0][
+            'id']  # get the Configuration Managers ID from response
+
+        payload = {
+            "configuration": [
+                {
+                    "system": {
+                        "users": {
+                            "0": {
+                                "password": new_password
+                            }
+                        }
+                    }
+                },
+                []
+            ]
+        }
+
+        ncm = self.session.patch(
+            '{0}/configuration_managers/{1}/'.format(self.base_url,
+                                                     str(config_man_id)),
+            data=json.dumps(payload))  # Patch indie config with new values
+        result = self.__return_handler(ncm.status_code, ncm.text, call_type)
+        return result
+
+    def set_ethernet_wan_ip(self, router_id: int, new_wan_ip: str,
+                            new_netmask: str = None, new_gateway: str = None):
+        """
+        This method sets the Ethernet WAN IP Address for a given router id.
+        :param router_id: ID of router to update
+        :param new_wan_ip: IP Address to assign to Ethernet WAN
+        :param new_netmask: Network Mask in dotted decimal notation (optional)
+        :param new_gateway: IP of gateway (optional)
+        :return:
+        """
+        call_type = 'Etheret WAN IP Address'
+
+        response = self.session.get(
+            '{0}/configuration_managers/?router.id={1}&fields=id'.format(
+                self.base_url,
+                str(router_id)))  # Get Configuration Managers ID
+        response = json.loads(response.content.decode(
+            "utf-8"))  # Decode the response and make it a dictionary
+        config_man_id = response['data'][0][
+            'id']  # get the Configuration Managers ID from response
+
+        ip_override = {
+            "ip_address": new_wan_ip
+        }
+
+        if new_netmask:
+            ip_override['netmask'] = new_netmask
+
+        if new_gateway:
+            ip_override['gateway'] = new_gateway
+
+        payload = {
+            "configuration": [
+                {
+                    "wan": {
+                        "rules2": {
+                            "0": {
+                                "ip_override": ip_override
+                            }
+                        }
+                    }
+                },
+                []
+            ]
+        }
+
+        ncm = self.session.patch(
+            '{0}/configuration_managers/{1}/'.format(self.base_url,
+                                                     str(config_man_id)),
+            data=json.dumps(payload))  # Patch indie config with new values
+        result = self.__return_handler(ncm.status_code, ncm.text, call_type)
+        return result
+
+    def add_custom_apn(self, router_id: int, new_carrier: str, new_apn: str):
+        """
+        This method adds a new APN to the Advanced APN configuration
+        :param router_id: ID of router to update
+        :param new_carrier: Home Carrier / PLMN
+        :param new_apn: APN
+        :return:
+        """
+        call_type = 'Custom APN'
+
+        response = self.session.get(
+            '{0}/configuration_managers/?router.id={1}&fields=id,configuration'.format(
+                self.base_url,
+                str(router_id)))  # Get Configuration Managers ID
+        response = json.loads(response.content.decode(
+            "utf-8"))  # Decode the response and make it a dictionary
+        config_man_id = response['data'][0][
+            'id']  # get the Configuration Managers ID from response
+
+        new_apn_id = 0
+        try:
+            if response['data'][0]['configuration'][0]['wan']:
+                if response['data'][0]['configuration'][0]['wan']['custom_apns']:
+                    new_apn_id = len(response['data'][0]['configuration'][0]['wan']['custom_apns'])
+        except KeyError:
+            pass
+
+        payload = {
+            "configuration": [
+                {
+                    "wan": {
+                        "custom_apns": {
+                            new_apn_id: {
+                                "apn": new_apn,
+                                "carrier": new_carrier
+                            }
+                        }
+                    }
+                },
+                []
+            ]
+        }
+
+        ncm = self.session.patch(
+            '{0}/configuration_managers/{1}/'.format(self.base_url,
+                                                     str(config_man_id)),
+            data=json.dumps(payload))  # Patch indie config with new values
+        result = self.__return_handler(ncm.status_code, ncm.text, call_type)
+        return result
